@@ -11,6 +11,8 @@
 #   /usr/src/redhat directory
 #
 
+EXTRA_OPTIONS="$1"
+
 SPECDIR=`rpm --eval %_specdir`
 SRCDIR=`rpm --eval %_sourcedir`
 
@@ -18,7 +20,8 @@ SRCDIR=`rpm --eval %_sourcedir`
 
 USERID=`id -u`
 GRPID=`id -g`
-VERSION='3.0.22'
+VERSION='3.0.23'
+REVISION=''
 SPECFILE="samba.spec"
 RPMVER=`rpm --version | awk '{print $3}'`
 RPM="rpmbuild"
@@ -36,10 +39,29 @@ case $RPMVER in
        ;;
 esac
 
-( cd ../../source; if [ -f Makefile ]; then make distclean; fi )
-( cd ../../.. ; chown -R ${USERID}.${GRPID} samba-${VERSION} )
+pushd .
+cd ../../source
+if [ -f Makefile ]; then 
+	make distclean
+fi
+popd
+
+pushd .
+cd ../../../
+chown -R ${USERID}.${GRPID} samba-${VERSION}${REVISION}
+if [ ! -d samba-${VERSION} ]; then
+	ln -s samba-${VERSION}${REVISION} samba-${VERSION} || exit 1
+fi
+echo -n "Creating samba-${VERSION}.tar.bz2 ... "
+tar --exclude=.svn -cf - samba-${VERSION}/. | bzip2 > ${SRCDIR}/samba-${VERSION}.tar.bz2
+echo "Done."
+if [ $? -ne 0 ]; then
+        echo "Build failed!"
+        exit 1
+fi
+
+popd
 
-( cd ../../.. ; tar --exclude=.svn -cf - samba-${VERSION}/. | bzip2 > ${SRCDIR}/samba-${VERSION}.tar.bz2 )
 
 ##
 ## copy additional source files
@@ -51,9 +73,9 @@ cp -p ${SPECFILE} ${SPECDIR}
 ##
 ## Build
 ##
-echo Getting Ready to build release package
+echo "$(basename $0): Getting Ready to build release package"
 cd ${SPECDIR}
-${RPM} -ba --clean --rmsource $SPECFILE
+${RPM} -ba --clean --rmsource $EXTRA_OPTIONS $SPECFILE
 
-echo Done.
+echo "$(basename $0): Done."
 
