@@ -20,7 +20,8 @@ SRCDIR=`rpm --eval %_sourcedir`
 
 USERID=`id -u`
 GRPID=`id -g`
-VERSION='3.0.22'
+VERSION='3.0.23'
+REVISION=''
 SPECFILE="samba3.spec"
 RPMVER=`rpm --version | awk '{print $3}'`
 RPM="rpm"
@@ -53,10 +54,29 @@ esac
 
 echo "RPM build command is \"$RPM\""
 
-( cd ../../source; if [ -f Makefile ]; then make distclean; fi )
-( cd ../../.. ; chown -R ${USERID}.${GRPID} samba-${VERSION} )
+pushd .
+cd ../../source
+if [ -f Makefile ]; then
+        make distclean
+fi
+popd
+
+pushd .
+cd ../../../
+chown -R ${USERID}.${GRPID} samba-${VERSION}${REVISION}
+if [ ! -d samba-${VERSION} ]; then
+        ln -s samba-${VERSION}${REVISION} samba-${VERSION} || exit 1
+fi
+echo -n "Creating samba-${VERSION}.tar.bz2 ... "
+tar --exclude=.svn -cf - samba-${VERSION}/. | bzip2 > ${SRCDIR}/samba-${VERSION}.tar.bz2
+echo "Done."
+if [ $? -ne 0 ]; then
+        echo "Build failed!"
+        exit 1
+fi
+
+popd
 
-( cd ../../.. ; tar --exclude=CVS -cf - samba-${VERSION}/. | bzip2 > ${SRCDIR}/samba-${VERSION}.tar.bz2 )
 
 /bin/cp -p filter-requires-samba_rh8.sh ${SRCDIR}
 /bin/cp -p filter-requires-samba_rh9.sh ${SRCDIR}
