@@ -169,19 +169,22 @@ static size_t interpret_long_filename(struct cli_state *cli, int level,char *p,f
 int cli_list_new(struct cli_state *cli,const char *Mask,uint16 attribute, 
 		 void (*fn)(const char *, file_info *, const char *, void *), void *state)
 {
-        int max_matches = 1366;
+#if 1
+	int max_matches = 1366; /* Match W2k - was 512. */
+#else
+	int max_matches = 512;
+#endif
 	int info_level;
 	char *p, *p2;
 	pstring mask;
 	file_info finfo;
 	int i;
-	char *tdl, *dirlist = NULL;
+	char *dirlist = NULL;
 	int dirlist_len = 0;
 	int total_received = -1;
 	BOOL First = True;
 	int ff_searchcount=0;
 	int ff_eos=0;
-	int ff_lastname=0;
 	int ff_dir_handle=0;
 	int loop_count = 0;
 	char *rparam=NULL, *rdata=NULL;
@@ -293,11 +296,9 @@ int cli_list_new(struct cli_state *cli,const char *Mask,uint16 attribute,
 			ff_dir_handle = SVAL(p,0);
 			ff_searchcount = SVAL(p,2);
 			ff_eos = SVAL(p,4);
-			ff_lastname = SVAL(p,8);
 		} else {
 			ff_searchcount = SVAL(p,0);
 			ff_eos = SVAL(p,2);
-			ff_lastname = SVAL(p,6);
 		}
 
 		if (ff_searchcount == 0) {
@@ -326,7 +327,7 @@ int cli_list_new(struct cli_state *cli,const char *Mask,uint16 attribute,
 			}
 		}
 
-		if (ff_lastname > 0) {
+		if (ff_searchcount > 0) {
 			pstrcpy(mask, finfo.name);
 		} else {
 			pstrcpy(mask,"");
@@ -334,15 +335,13 @@ int cli_list_new(struct cli_state *cli,const char *Mask,uint16 attribute,
 
 		/* grab the data for later use */
 		/* and add them to the dirlist pool */
-		tdl = SMB_REALLOC(dirlist,dirlist_len + data_len);
+		dirlist = SMB_REALLOC(dirlist,dirlist_len + data_len);
 
-		if (!tdl) {
+		if (!dirlist) {
 			DEBUG(0,("cli_list_new: Failed to expand dirlist\n"));
 			SAFE_FREE(rdata);
 			SAFE_FREE(rparam);
 			break;
-		} else {
-			dirlist = tdl;
 		}
 
 		memcpy(dirlist+dirlist_len,p,data_len);
@@ -417,7 +416,7 @@ int cli_list_old(struct cli_state *cli,const char *Mask,uint16 attribute,
 	int num_asked = (cli->max_xmit - 100)/DIR_STRUCT_SIZE;
 	int num_received = 0;
 	int i;
-	char *tdl, *dirlist = NULL;
+	char *dirlist = NULL;
 	pstring mask;
 	
 	ZERO_ARRAY(status);
@@ -462,14 +461,11 @@ int cli_list_old(struct cli_state *cli,const char *Mask,uint16 attribute,
 
 		first = False;
 
-		tdl = SMB_REALLOC(dirlist,(num_received + received)*DIR_STRUCT_SIZE);
-
-		if (!tdl) {
+		dirlist = SMB_REALLOC(dirlist,(num_received + received)*DIR_STRUCT_SIZE);
+		if (!dirlist) {
 			DEBUG(0,("cli_list_old: failed to expand dirlist"));
-			SAFE_FREE(dirlist);
 			return 0;
 		}
-		else dirlist = tdl;
 
 		p = smb_buf(cli->inbuf) + 3;
 
