@@ -26,8 +26,6 @@
 #include "includes.h"
 
 extern struct in_addr loopback_ip;
-extern int ClientNMB;
-extern int ClientDGRAM;
 extern int global_nmb_port;
 
 /* This is the broadcast subnets database. */
@@ -51,34 +49,6 @@ static void add_subnet(struct subnet_record *subrec)
 	DLIST_ADD(subnetlist, subrec);
 }
 
-/* ************************************************************************** **
- * Comparison routine for ordering the splay-tree based namelists assoicated
- * with each subnet record.
- *
- *  Input:  Item  - Pointer to the comparison key.
- *          Node  - Pointer to a node the splay tree.
- *
- *  Output: The return value will be <0 , ==0, or >0 depending upon the
- *          ordinal relationship of the two keys.
- *
- * ************************************************************************** **
- */
-static int namelist_entry_compare( ubi_trItemPtr Item, ubi_trNodePtr Node )
-{
-	struct name_record *NR = (struct name_record *)Node;
-
-	if( DEBUGLVL( 10 ) ) {
-		struct nmb_name *Iname = (struct nmb_name *)Item;
-
-		Debug1( "nmbd_subnetdb:namelist_entry_compare()\n" );
-		Debug1( "%d == memcmp( \"%s\", \"%s\", %d )\n",
-			memcmp( Item, &(NR->name), sizeof(struct nmb_name) ),
-			nmb_namestr(Iname), nmb_namestr(&NR->name), (int)sizeof(struct nmb_name) );
-	}
-
-	return( memcmp( Item, &(NR->name), sizeof(struct nmb_name) ) ); 
-}
-
 /****************************************************************************
 stop listening on a subnet
 we don't free the record as we don't have proper reference counting for it
@@ -87,8 +57,6 @@ yet and it may be in use by a response record
 
 void close_subnet(struct subnet_record *subrec)
 {
-	DLIST_REMOVE(subnetlist, subrec);
-
 	if (subrec->dgram_sock != -1) {
 		close(subrec->dgram_sock);
 		subrec->dgram_sock = -1;
@@ -97,6 +65,8 @@ void close_subnet(struct subnet_record *subrec)
 		close(subrec->nmb_sock);
 		subrec->nmb_sock = -1;
 	}
+
+	DLIST_REMOVE(subnetlist, subrec);
 }
 
 /****************************************************************************
@@ -146,6 +116,10 @@ static struct subnet_record *make_subnet(const char *name, enum subnet_type type
 		/* Make sure we can broadcast from these sockets. */
 		set_socket_options(nmb_sock,"SO_BROADCAST");
 		set_socket_options(dgram_sock,"SO_BROADCAST");
+
+		/* Set them non-blocking. */
+		set_blocking(nmb_sock, False);
+		set_blocking(dgram_sock, False);
 	}
 
 	subrec = SMB_MALLOC_P(struct subnet_record);
@@ -156,10 +130,7 @@ static struct subnet_record *make_subnet(const char *name, enum subnet_type type
 		return(NULL);
 	}
   
-	memset( (char *)subrec, '\0', sizeof(*subrec) );
-	(void)ubi_trInitTree( subrec->namelist,
-			namelist_entry_compare,
-			ubi_trOVERWRITE );
+	ZERO_STRUCTP(subrec);
 
 	if((subrec->subnet_name = SMB_STRDUP(name)) == NULL) {
 		DEBUG(0,("make_subnet: malloc fail for subnet name !\n"));
@@ -232,6 +203,11 @@ BOOL create_subnets(void)
 	for (i = 0 ; i < num_interfaces; i++) {
 		struct interface *iface = get_interface(i);
 
+		if (!iface) {
+			DEBUG(2,("create_subnets: can't get interface %d.\n", i ));
+			continue;
+		}
+
 		/*
 		 * We don't want to add a loopback interface, in case
 		 * someone has added 127.0.0.1 for smbd, nmbd needs to
@@ -249,7 +225,13 @@ BOOL create_subnets(void)
 
 	if (lp_we_are_a_wins_server()) {
 		/* Pick the first interface ip address as the WINS server ip. */
-		unicast_ip = *iface_n_ip(0);
+		struct in_addr *nip = iface_n_ip(0);
+
+		if (!nip) {
+			return False;
+		}
+
+		unicast_ip = *nip;
 	} else {
 		/* note that we do not set the wins server IP here. We just
 			set it at zero and let the wins registration code cope
