@@ -54,7 +54,7 @@ static REGISTRY_KEY *find_regkey_index_by_hnd(pipes_struct *p, POLICY_HND *hnd)
 {
 	REGISTRY_KEY *regkey = NULL;
 
-	if(!find_policy_by_hnd(p,hnd,(void **)&regkey)) {
+	if(!find_policy_by_hnd(p,hnd,(void **)(void *)&regkey)) {
 		DEBUG(2,("find_regkey_index_by_hnd: Registry Key not found: "));
 		return NULL;
 	}
@@ -835,8 +835,10 @@ static WERROR restore_registry_key ( REGISTRY_KEY *krecord, const char *fname )
 	/* get the rootkey from the regf file and then load the tree
 	   via recursive calls */
 	   
-	if ( !(rootkey = regfio_rootkey( regfile )) )
+	if ( !(rootkey = regfio_rootkey( regfile )) ) {
+		regfio_close( regfile );
 		return WERR_REG_FILE_INVALID;
+	}
 	
 	result = reg_load_tree( regfile, krecord->name, rootkey );
 		
@@ -1197,7 +1199,7 @@ WERROR _reg_set_value(pipes_struct *p, REG_Q_SET_VALUE  *q_u, REG_R_SET_VALUE *r
 WERROR _reg_delete_key(pipes_struct *p, REG_Q_DELETE_KEY  *q_u, REG_R_DELETE_KEY *r_u)
 {
 	REGISTRY_KEY *parent = find_regkey_index_by_hnd(p, &q_u->handle);
-	REGISTRY_KEY *newparentinfo;
+	REGISTRY_KEY *newparentinfo = NULL;
 	POLICY_HND newparent_handle;
 	REGSUBKEY_CTR *subkeys;
 	BOOL write_result;
