@@ -94,7 +94,7 @@ void make_dir_struct(char *buf, const char *mask, const char *fname,SMB_OFF_T si
 
 	memset(buf+21,'\0',DIR_STRUCT_SIZE-21);
 	SCVAL(buf,21,mode);
-	put_dos_date(buf,22,date);
+	srv_put_dos_date(buf,22,date);
 	SSVAL(buf,26,size & 0xFFFF);
 	SSVAL(buf,28,(size >> 16)&0xFFFF);
 	/* We only uppercase if FLAGS2_LONG_PATH_COMPONENTS is zero in the input buf.
@@ -382,7 +382,7 @@ static void dptr_close_oldest(BOOL old)
 ****************************************************************************/
 
 int dptr_create(connection_struct *conn, pstring path, BOOL old_handle, BOOL expect_close,uint16 spid,
-		const char *wcard, uint32 attr)
+		const char *wcard, BOOL wcard_has_wild, uint32 attr)
 {
 	struct dptr_struct *dptr = NULL;
 	struct smb_Dir *dir_hnd;
@@ -500,7 +500,7 @@ int dptr_create(connection_struct *conn, pstring path, BOOL old_handle, BOOL exp
 	if (lp_posix_pathnames() || (wcard[0] == '.' && wcard[1] == 0)) {
 		dptr->has_wild = True;
 	} else {
-		dptr->has_wild = ms_has_wild(wcard);
+		dptr->has_wild = wcard_has_wild;
 	}
 
 	dptr->attr = attr;
@@ -636,12 +636,16 @@ BOOL dptr_SearchDir(struct dptr_struct *dptr, const char *name, long *poffset, S
 		return False;
 	}
 
-	if (SearchDir(dptr->dir_hnd, name, poffset)) {
-		if (is_visible_file(dptr->conn, dptr->path, name, pst, True)) {
-			return True;
-		}
-	}
-	return False;
+	return SearchDir(dptr->dir_hnd, name, poffset);
+}
+
+/****************************************************************************
+ Add the name we're returning into the underlying cache.
+****************************************************************************/
+
+void dptr_DirCacheAdd(struct dptr_struct *dptr, const char *name, long offset)
+{
+	DirCacheAdd(dptr->dir_hnd, name, offset);
 }
 
 /****************************************************************************
@@ -710,7 +714,7 @@ struct dptr_struct *dptr_fetch_lanman2(int dptr_num)
 }
 
 /****************************************************************************
- Check a filetype for being valid.
+ Check that a file matches a particular file type.
 ****************************************************************************/
 
 BOOL dir_check_ftype(connection_struct *conn, uint32 mode, uint32 dirtype)
@@ -802,7 +806,7 @@ BOOL get_dir_entry(connection_struct *conn,char *mask,uint32 dirtype, pstring fn
 			*mode = dos_mode(conn,pathreal,&sbuf);
 
 			if (!dir_check_ftype(conn,*mode,dirtype)) {
-				DEBUG(5,("[%s] attribs didn't match %x\n",filename,(unsigned int)dirtype));
+				DEBUG(5,("[%s] attribs 0x%x didn't match 0x%x\n",filename,(unsigned int)*mode,(unsigned int)dirtype));
 				continue;
 			}
 
@@ -812,6 +816,8 @@ BOOL get_dir_entry(connection_struct *conn,char *mask,uint32 dirtype, pstring fn
 			DEBUG(3,("get_dir_entry mask=[%s] found %s fname=%s\n",mask, pathreal,fname));
 
 			found = True;
+
+			DirCacheAdd(conn->dirptr->dir_hnd, dname, curoff);
 		}
 	}
 
@@ -843,6 +849,8 @@ static BOOL user_can_read_file(connection_struct *conn, char *name, SMB_STRUCT_S
 
 	/* If we can't stat it does not show it */
 	if (!VALID_STAT(*pst) && (SMB_VFS_STAT(conn, name, pst) != 0)) {
+		DEBUG(10,("user_can_read_file: SMB_VFS_STAT failed for file %s with error %s\n",
+			name, strerror(errno) ));
 		return False;
 	}
 
@@ -866,7 +874,7 @@ static BOOL user_can_read_file(connection_struct *conn, char *name, SMB_STRUCT_S
 	/* Get NT ACL -allocated in main loop talloc context. No free needed here. */
 	sd_size = SMB_VFS_FGET_NT_ACL(fsp, fsp->fh->fd,
 			(OWNER_SECURITY_INFORMATION|GROUP_SECURITY_INFORMATION|DACL_SECURITY_INFORMATION), &psd);
-	close_file(fsp, True);
+	close_file(fsp, NORMAL_CLOSE);
 
 	/* No access if SD get failed. */
 	if (!sd_size) {
@@ -929,7 +937,7 @@ static BOOL user_can_write_file(connection_struct *conn, char *name, SMB_STRUCT_
 	/* Get NT ACL -allocated in main loop talloc context. No free needed here. */
 	sd_size = SMB_VFS_FGET_NT_ACL(fsp, fsp->fh->fd,
 			(OWNER_SECURITY_INFORMATION|GROUP_SECURITY_INFORMATION|DACL_SECURITY_INFORMATION), &psd);
-	close_file(fsp, False);
+	close_file(fsp, NORMAL_CLOSE);
 
 	/* No access if SD get failed. */
 	if (!sd_size)
@@ -981,6 +989,7 @@ BOOL is_visible_file(connection_struct *conn, const char *dir_path, const char *
 
 	/* If it's a vetoed file, pretend it doesn't even exist */
 	if (use_veto && IS_VETO_PATH(conn, name)) {
+		DEBUG(10,("is_visible_file: file %s is vetoed.\n", name ));
 		return False;
 	}
 
@@ -992,16 +1001,19 @@ BOOL is_visible_file(connection_struct *conn, const char *dir_path, const char *
 		}
 		/* Honour _hide unreadable_ option */
 		if (hide_unreadable && !user_can_read_file(conn, entry, pst)) {
+			DEBUG(10,("is_visible_file: file %s is unreadable.\n", entry ));
 			SAFE_FREE(entry);
 			return False;
 		}
 		/* Honour _hide unwriteable_ option */
 		if (hide_unwriteable && !user_can_write_file(conn, entry, pst)) {
+			DEBUG(10,("is_visible_file: file %s is unwritable.\n", entry ));
 			SAFE_FREE(entry);
 			return False;
 		}
 		/* Honour _hide_special_ option */
 		if (hide_special && file_is_special(conn, entry, pst)) {
+			DEBUG(10,("is_visible_file: file %s is special.\n", entry ));
 			SAFE_FREE(entry);
 			return False;
 		}
@@ -1109,21 +1121,15 @@ const char *ReadDirName(struct smb_Dir *dirp, long *poffset)
 	}
 
 	while ((n = vfs_readdirname(conn, dirp->dir))) {
-		struct name_cache_entry *e;
 		/* Ignore . and .. - we've already returned them. */
 		if (*n == '.') {
 			if ((n[1] == '\0') || (n[1] == '.' && n[2] == '\0')) {
 				continue;
 			}
 		}
-		dirp->offset = SMB_VFS_TELLDIR(conn, dirp->dir);
-		dirp->name_cache_index = (dirp->name_cache_index+1) % NAME_CACHE_SIZE;
-		e = &dirp->name_cache[dirp->name_cache_index];
-		SAFE_FREE(e->name);
-		e->name = SMB_STRDUP(n);
-		*poffset = e->offset= dirp->offset;
+		*poffset = dirp->offset = SMB_VFS_TELLDIR(conn, dirp->dir);
 		dirp->file_number++;
-		return e->name;
+		return n;
 	}
 	*poffset = dirp->offset = END_OF_DIRECTORY_OFFSET;
 	return NULL;
@@ -1183,6 +1189,21 @@ long TellDir(struct smb_Dir *dirp)
 	return(dirp->offset);
 }
 
+/*******************************************************************
+ Add an entry into the dcache.
+********************************************************************/
+
+void DirCacheAdd(struct smb_Dir *dirp, const char *name, long offset)
+{
+	struct name_cache_entry *e;
+
+	dirp->name_cache_index = (dirp->name_cache_index+1) % NAME_CACHE_SIZE;
+	e = &dirp->name_cache[dirp->name_cache_index];
+	SAFE_FREE(e->name);
+	e->name = SMB_STRDUP(name);
+	e->offset = offset;
+}
+
 /*******************************************************************
  Find an entry by name. Leave us at the offset after it.
  Don't check for veto or invisible files.
