@@ -114,7 +114,8 @@ static BOOL parse_processed_dfs_path(char* pathname, struct dfs_path *pdp, BOOL
 
 	/* rest is reqpath */
 	if (allow_wcards) {
-		check_path_syntax_wcard(pdp->reqpath, p+1);
+		BOOL path_contains_wcard;
+		check_path_syntax_wcard(pdp->reqpath, p+1, &path_contains_wcard);
 	} else {
 		check_path_syntax(pdp->reqpath, p+1);
 	}
@@ -145,7 +146,7 @@ static BOOL create_conn_struct(connection_struct *conn, int snum, char *path)
                 return False;
         }
 	
-	string_set(&conn->connectpath, connpath);
+	set_conn_connectpath(conn, connpath);
 
 	if (!smbd_vfs_init(conn)) {
 		DEBUG(0,("create_conn_struct: smbd_vfs_init failed.\n"));
@@ -202,7 +203,8 @@ static BOOL parse_symlink(TALLOC_CTX *ctx, char *buf, struct referral **preflist
 	}
 
 	/* parse out the alternate paths */
-	while(((alt_path[count] = strtok(NULL,",")) != NULL) && count<MAX_REFERRAL_COUNT) {
+	while((count<MAX_REFERRAL_COUNT) &&
+	      ((alt_path[count] = strtok(NULL,",")) != NULL)) {
 		count++;
 	}
 
@@ -418,7 +420,10 @@ BOOL dfs_redirect( pstring pathname, connection_struct *conn, BOOL search_wcard_
 		return False;
 	}
 	
-	if (!strequal(dp.servicename, lp_servicename(SNUM(conn)) )) {
+	if ( !( strequal(dp.servicename, lp_servicename(SNUM(conn))) 
+		|| ( strequal(dp.servicename, HOMES_NAME) 
+		     && strequal(lp_servicename(SNUM(conn)), get_current_username()) )) ) 
+	{
 		return False;
 	}
 
@@ -500,13 +505,10 @@ BOOL get_referred_path(TALLOC_CTX *ctx, char *pathname, struct junction_map *juc
 	parse_dfs_path(pathname, &dp);
 
 	/* Verify hostname in path */
-	if ( !strequal(get_local_machine_name(), dp.hostname) ) {
-		/* Hostname mismatch, check if one of our IP addresses */
-		if (!ismyip(*interpret_addr2(dp.hostname))) {
-			DEBUG(3, ("get_referred_path: Invalid hostname %s in path %s\n",
-				dp.hostname, pathname));
-			return False;
-		}
+	if (!is_myname_or_ipaddr(dp.hostname)) {
+		DEBUG(3, ("get_referred_path: Invalid hostname %s in path %s\n",
+			dp.hostname, pathname));
+		return False;
 	}
 
 	pstrcpy(jucn->service_name, dp.servicename);
@@ -608,7 +610,7 @@ static int setup_ver2_dfs_referral(char *pathname, char **ppdata,
 
 	DEBUG(10,("setting up version2 referral\nRequested path:\n"));
 
-	requestedpathlen = rpcstr_push(uni_requestedpath, pathname, -1,
+	requestedpathlen = rpcstr_push(uni_requestedpath, pathname, sizeof(pstring),
 				       STR_TERMINATE);
 
 	if (DEBUGLVL(10)) {
@@ -642,9 +644,8 @@ static int setup_ver2_dfs_referral(char *pathname, char **ppdata,
 	if(pdata == NULL) {
 		DEBUG(0,("malloc failed for Realloc!\n"));
 		return -1;
-	} else {
-		*ppdata = pdata;
 	}
+	*ppdata = pdata;
 
 	/* copy in the dfs requested paths.. required for offset calculations */
 	memcpy(pdata+uni_reqpathoffset1,uni_requestedpath,requestedpathlen);
@@ -680,7 +681,7 @@ static int setup_ver2_dfs_referral(char *pathname, char **ppdata,
 		SSVAL(pdata,offset+18,uni_reqpathoffset2-offset);
 		/* copy referred path into current offset */
 		unilen = rpcstr_push(pdata+uni_curroffset, ref->alternate_path,
-				     -1, STR_UNICODE);
+				     sizeof(pstring), STR_UNICODE);
 
 		SSVAL(pdata,offset+20,uni_curroffset-offset);
 
@@ -709,7 +710,7 @@ static int setup_ver3_dfs_referral(char *pathname, char **ppdata,
 	
 	DEBUG(10,("setting up version3 referral\n"));
 
-	reqpathlen = rpcstr_push(uni_reqpath, pathname, -1, STR_TERMINATE);
+	reqpathlen = rpcstr_push(uni_reqpath, pathname, sizeof(pstring), STR_TERMINATE);
 	
 	if (DEBUGLVL(10)) {
 	    dump_data(0, (char *) uni_reqpath,reqpathlen);
@@ -728,9 +729,8 @@ static int setup_ver3_dfs_referral(char *pathname, char **ppdata,
 	if(pdata == NULL) {
 		DEBUG(0,("version3 referral setup: malloc failed for Realloc!\n"));
 		return -1;
-	} else {
-		*ppdata = pdata;
 	}
+	*ppdata = pdata;
 
 	/* create the header */
 	SSVAL(pdata,0,consumedcnt * 2); /* path consumed */
@@ -765,7 +765,7 @@ static int setup_ver3_dfs_referral(char *pathname, char **ppdata,
 		SSVAL(pdata,offset+14,uni_reqpathoffset2-offset);
 		/* copy referred path into current offset */
 		unilen = rpcstr_push(pdata+uni_curroffset,ref->alternate_path,
-				     -1, STR_UNICODE | STR_TERMINATE);
+				     sizeof(pstring), STR_UNICODE | STR_TERMINATE);
 		SSVAL(pdata,offset+16,uni_curroffset-offset);
 		/* copy 0x10 bytes of 00's in the ServiceSite GUID */
 		memset(pdata+offset+18,'\0',16);
@@ -875,13 +875,10 @@ BOOL create_junction(char *pathname, struct junction_map *jucn)
         parse_dfs_path(pathname,&dp);
 
         /* check if path is dfs : validate first token */
-        if ( !strequal(get_local_machine_name(),dp.hostname) ) {
-		/* Hostname mismatch, check if one of our IP addresses */
-		if (!ismyip(*interpret_addr2(dp.hostname))) {
-			DEBUG(4,("create_junction: Invalid hostname %s in dfs path %s\n",
-				dp.hostname, pathname));
-			return False;
-		}
+	if (!is_myname_or_ipaddr(dp.hostname)) {
+		DEBUG(4,("create_junction: Invalid hostname %s in dfs path %s\n",
+			dp.hostname, pathname));
+		return False;
 	}
 
 	/* Check for a non-DFS share */
@@ -1093,13 +1090,19 @@ out:
 int enum_msdfs_links(TALLOC_CTX *ctx, struct junction_map *jucn, int jn_max)
 {
 	int i=0;
+	int sharecount = 0;
 	int jn_count = 0;
 
 	if(!lp_host_msdfs()) {
 		return 0;
 	}
 
-	for(i=0;i < lp_numservices() && (jn_max - jn_count) > 0;i++) {
+	/* Ensure all the usershares are loaded. */
+	become_root();
+	sharecount = load_usershare_shares();
+	unbecome_root();
+
+	for(i=0;i < sharecount && (jn_max - jn_count) > 0;i++) {
 		if(lp_msdfs_root(i)) {
 			jn_count += form_junctions(ctx, i,jucn,jn_max - jn_count);
 		}
