@@ -22,6 +22,7 @@
 
 #include "includes.h"
 
+extern struct generic_mapping file_generic_mapping;
 extern struct current_user current_user;
 extern userdom_struct current_user_info;
 extern uint16 global_smbpid;
@@ -73,23 +74,6 @@ int fd_close(struct connection_struct *conn,
 	return fd_close_posix(conn, fsp);
 }
 
-
-/****************************************************************************
- Check a filename for the pipe string.
-****************************************************************************/
-
-static void check_for_pipe(const char *fname)
-{
-	/* special case of pipe opens */
-	char s[10];
-	StrnCpy(s,fname,sizeof(s)-1);
-	strlower_m(s);
-	if (strstr(s,"pipe/")) {
-		DEBUG(3,("Rejecting named pipe open for %s\n",fname));
-		set_saved_error_triple(ERRSRV, ERRaccess, NT_STATUS_ACCESS_DENIED);
-	}
-}
-
 /****************************************************************************
  Change the ownership of a file to that of the parent directory.
  Do this by fd if possible.
@@ -226,7 +210,6 @@ static BOOL open_file(files_struct *fsp,
 		/* It's a read-only share - fail if we wanted to write. */
 		if(accmode != O_RDONLY) {
 			DEBUG(3,("Permission denied opening %s\n",fname));
-			check_for_pipe(fname);
 			return False;
 		} else if(flags & O_CREAT) {
 			/* We don't want to write - but we must make sure that
@@ -262,7 +245,7 @@ static BOOL open_file(files_struct *fsp,
 
 		/*
 		 * We can't actually truncate here as the file may be locked.
-		 * open_file_shared will take care of the truncate later. JRA.
+		 * open_file_ntcreate will take care of the truncate later. JRA.
 		 */
 
 		local_flags &= ~O_TRUNC;
@@ -292,7 +275,6 @@ static BOOL open_file(files_struct *fsp,
 			DEBUG(3,("Error opening file %s (%s) (local_flags=%d) "
 				 "(flags=%d)\n",
 				 fname,strerror(errno),local_flags,flags));
-			check_for_pipe(fname);
 			return False;
 		}
 
@@ -617,7 +599,7 @@ static BOOL is_delete_request(files_struct *fsp) {
 }
 
 /*
- * 1) No files open at all: Grant whatever the client wants.
+ * 1) No files open at all or internal open: Grant whatever the client wants.
  *
  * 2) Exclusive (or batch) oplock around: If the requested access is a delete
  *    request, break if the oplock around is a batch oplock. If it's another
@@ -626,45 +608,59 @@ static BOOL is_delete_request(files_struct *fsp) {
  * 3) Only level2 around: Grant level2 and do nothing else.
  */
 
-static BOOL delay_for_oplocks(struct share_mode_lock *lck, files_struct *fsp)
+static BOOL delay_for_oplocks(struct share_mode_lock *lck,
+				files_struct *fsp,
+				int pass_number,
+				int oplock_request)
 {
 	int i;
 	struct share_mode_entry *exclusive = NULL;
+	BOOL valid_entry = False;
 	BOOL delay_it = False;
 	BOOL have_level2 = False;
 
-	if (is_stat_open(fsp->access_mask)) {
+	if ((oplock_request & INTERNAL_OPEN_ONLY) || is_stat_open(fsp->access_mask)) {
 		fsp->oplock_type = NO_OPLOCK;
 		return False;
 	}
 
-	if (lck->num_share_modes == 0) {
-		/* No files open at all: Directly grant whatever the client
-		 * wants. */
-
-		if (fsp->oplock_type == NO_OPLOCK) {
-			/* Store a level2 oplock, but don't tell the client */
-			fsp->oplock_type = FAKE_LEVEL_II_OPLOCK;
-		}
-		return False;
-	}
-
 	for (i=0; i<lck->num_share_modes; i++) {
 
 		if (!is_valid_share_mode_entry(&lck->share_modes[i])) {
 			continue;
 		}
 
-		if (EXCLUSIVE_OPLOCK_TYPE(lck->share_modes[i].op_type)) {
-			SMB_ASSERT(exclusive == NULL);			
-			exclusive = &lck->share_modes[i];
+		/* At least one entry is not an invalid or deferred entry. */
+		valid_entry = True;
+
+		if (pass_number == 1) {
+			if (BATCH_OPLOCK_TYPE(lck->share_modes[i].op_type)) {
+				SMB_ASSERT(exclusive == NULL);			
+				exclusive = &lck->share_modes[i];
+			}
+		} else {
+			if (EXCLUSIVE_OPLOCK_TYPE(lck->share_modes[i].op_type)) {
+				SMB_ASSERT(exclusive == NULL);			
+				exclusive = &lck->share_modes[i];
+			}
 		}
 
 		if (lck->share_modes[i].op_type == LEVEL_II_OPLOCK) {
+			SMB_ASSERT(exclusive == NULL);			
 			have_level2 = True;
 		}
 	}
 
+	if (!valid_entry) {
+		/* All entries are placeholders or deferred.
+		 * Directly grant whatever the client wants. */
+		if (fsp->oplock_type == NO_OPLOCK) {
+			/* Store a level2 oplock, but don't tell the client */
+			fsp->oplock_type = FAKE_LEVEL_II_OPLOCK;
+		}
+		return False;
+	}
+
 	if (exclusive != NULL) { /* Found an exclusive oplock */
 		SMB_ASSERT(!have_level2);
 		delay_it = is_delete_request(fsp) ?
@@ -672,7 +668,8 @@ static BOOL delay_for_oplocks(struct share_mode_lock *lck, files_struct *fsp)
 	}
 
 	if (EXCLUSIVE_OPLOCK_TYPE(fsp->oplock_type)) {
-		/* We can at most grant level2 */
+		/* We can at most grant level2 as there are other
+		 * level2 or NO_OPLOCK entries. */
 		fsp->oplock_type = LEVEL_II_OPLOCK;
 	}
 
@@ -683,17 +680,29 @@ static BOOL delay_for_oplocks(struct share_mode_lock *lck, files_struct *fsp)
 
 	if (delay_it) {
 		BOOL ret;
+		char msg[MSG_SMB_SHARE_MODE_ENTRY_SIZE];
+
 		DEBUG(10, ("Sending break request to PID %s\n",
 			   procid_str_static(&exclusive->pid)));
 		exclusive->op_mid = get_current_mid();
+
+		/* Create the message. */
+		share_mode_entry_to_message(msg, exclusive);
+
+		/* Add in the FORCE_OPLOCK_BREAK_TO_NONE bit in the message if set. We don't
+		   want this set in the share mode struct pointed to by lck. */
+
+		if (oplock_request & FORCE_OPLOCK_BREAK_TO_NONE) {
+			SSVAL(msg,6,exclusive->op_type | FORCE_OPLOCK_BREAK_TO_NONE);
+		}
+
 		become_root();
 		ret = message_send_pid(exclusive->pid, MSG_SMB_BREAK_REQUEST,
-				       exclusive, sizeof(*exclusive), True);
+				       msg, MSG_SMB_SHARE_MODE_ENTRY_SIZE, True);
 		unbecome_root();
 		if (!ret) {
 			DEBUG(3, ("Could not send oplock break message\n"));
 		}
-		file_free(fsp);
 	}
 
 	return delay_it;
@@ -732,7 +741,7 @@ static void defer_open(struct share_mode_lock *lck,
 		if (procid_is_me(&e->pid) && (e->op_mid == mid)) {
 			DEBUG(0, ("Trying to defer an already deferred "
 				  "request: mid=%d, exiting\n", mid));
-			exit_server("exiting");
+			exit_server("attempt to defer a deferred request");
 		}
 	}
 
@@ -746,7 +755,7 @@ static void defer_open(struct share_mode_lock *lck,
 
 	if (!push_deferred_smb_message(mid, request_time, timeout,
 				       (char *)state, sizeof(*state))) {
-		exit_server("push_deferred_smb_message failed\n");
+		exit_server("push_deferred_smb_message failed");
 	}
 	add_deferred_open(lck, mid, request_time, state->dev, state->inode);
 
@@ -1032,14 +1041,41 @@ BOOL map_open_params_to_ntcreate(const char *fname, int deny_mode, int open_func
 
 }
 
-/* Map generic permissions to file object specific permissions */
-                                                                                                               
-struct generic_mapping file_generic_mapping = {
-	FILE_GENERIC_READ,
-	FILE_GENERIC_WRITE,
-	FILE_GENERIC_EXECUTE,
-	FILE_GENERIC_ALL
-};
+static void schedule_defer_open(struct share_mode_lock *lck, struct timeval request_time)
+{
+	struct deferred_open_record state;
+
+	/* This is a relative time, added to the absolute
+	   request_time value to get the absolute timeout time.
+	   Note that if this is the second or greater time we enter
+	   this codepath for this particular request mid then
+	   request_time is left as the absolute time of the *first*
+	   time this request mid was processed. This is what allows
+	   the request to eventually time out. */
+
+	struct timeval timeout;
+
+	/* Normally the smbd we asked should respond within
+	 * OPLOCK_BREAK_TIMEOUT seconds regardless of whether
+	 * the client did, give twice the timeout as a safety
+	 * measure here in case the other smbd is stuck
+	 * somewhere else. */
+
+	timeout = timeval_set(OPLOCK_BREAK_TIMEOUT*2, 0);
+
+	/* Nothing actually uses state.delayed_for_oplocks
+	   but it's handy to differentiate in debug messages
+	   between a 30 second delay due to oplock break, and
+	   a 1 second delay for share mode conflicts. */
+
+	state.delayed_for_oplocks = True;
+	state.dev = lck->dev;
+	state.inode = lck->ino;
+
+	if (!request_timed_out(request_time, timeout)) {
+		defer_open(lck, request_time, timeout, &state);
+	}
+}
 
 /****************************************************************************
  Open a file with a share mode.
@@ -1061,7 +1097,6 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 	int flags2=0;
 	BOOL file_existed = VALID_STAT(*psbuf);
 	BOOL def_acl = False;
-	BOOL internal_only_open = False;
 	SMB_DEV_T dev = 0;
 	SMB_INO_T inode = 0;
 	BOOL fsp_open = False;
@@ -1072,7 +1107,6 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 	uint32 existing_dos_attributes = 0;
 	struct pending_message_list *pml = NULL;
 	uint16 mid = get_current_mid();
-	BOOL delayed_for_oplocks = False;
 	struct timeval request_time = timeval_zero();
 	struct share_mode_lock *lck = NULL;
 	NTSTATUS status;
@@ -1104,36 +1138,23 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 		   create_disposition, create_options, unx_mode,
 		   oplock_request));
 
-	if (oplock_request == INTERNAL_OPEN_ONLY) {
-		internal_only_open = True;
-		oplock_request = 0;
-	}
-
 	if ((pml = get_open_deferred_message(mid)) != NULL) {
 		struct deferred_open_record *state =
 			(struct deferred_open_record *)pml->private_data.data;
 
+		/* Remember the absolute time of the original
+		   request with this mid. We'll use it later to
+		   see if this has timed out. */
+
 		request_time = pml->request_time;
-		delayed_for_oplocks = state->delayed_for_oplocks;
 
-		/* There could be a race condition where the dev/inode pair
-		   has changed since we deferred the message. If so, just
-		   remove the deferred open entry and return sharing
-		   violation. */
-
-		/* If the timeout value is non-zero, we need to just return
-		   sharing violation. Don't retry the open as we were not
-		   notified of a close and we don't want to trigger another
-		   spurious oplock break. */
-
-		/* Now remove the deferred open entry under lock. */
-		lck = get_share_mode_lock(NULL, state->dev, state->inode,
-					  fname);
+		/* Remove the deferred open entry under lock. */
+		lck = get_share_mode_lock(NULL, state->dev, state->inode, NULL, NULL);
 		if (lck == NULL) {
 			DEBUG(0, ("could not get share mode lock\n"));
 		} else {
 			del_deferred_open_entry(lck, mid);
-			talloc_destroy(lck);
+			TALLOC_FREE(lck);
 		}
 
 		/* Ensure we don't reprocess this message. */
@@ -1152,7 +1173,8 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 	/* ignore any oplock requests if oplocks are disabled */
 	if (!lp_oplocks(SNUM(conn)) || global_client_failed_oplock_break ||
 	    IS_VETO_OPLOCK_PATH(conn, fname)) {
-		oplock_request = 0;
+		/* Mask off everything except the private Samba bits. */
+		oplock_request &= SAMBA_PRIVATE_OPLOCK_MASK;
 	}
 
 	/* this is for OS/2 long file names - say we don't support them */
@@ -1289,7 +1311,7 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 	 */
 
 #if defined(O_SYNC)
-	if (create_options & FILE_WRITE_THROUGH) {
+	if ((create_options & FILE_WRITE_THROUGH) && lp_strict_sync(SNUM(conn))) {
 		flags2 |= O_SYNC;
 	}
 #endif /* O_SYNC */
@@ -1327,53 +1349,33 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 	fsp->share_access = share_access;
 	fsp->fh->private_options = create_options;
 	fsp->access_mask = access_mask;
-	fsp->oplock_type = oplock_request;
+	/* Ensure no SAMBA_PRIVATE bits can be set. */
+	fsp->oplock_type = (oplock_request & ~SAMBA_PRIVATE_OPLOCK_MASK);
 
 	if (timeval_is_zero(&request_time)) {
 		request_time = fsp->open_time;
 	}
 
 	if (file_existed) {
-
 		dev = psbuf->st_dev;
 		inode = psbuf->st_ino;
 
-		lck = get_share_mode_lock(NULL, dev, inode, fname);
+		lck = get_share_mode_lock(NULL, dev, inode,
+					conn->connectpath,
+					fname);
 
 		if (lck == NULL) {
+			file_free(fsp);
 			DEBUG(0, ("Could not get share mode lock\n"));
 			set_saved_ntstatus(NT_STATUS_SHARING_VIOLATION);
 			return NULL;
 		}
 
-		if (delay_for_oplocks(lck, fsp)) {
-			struct deferred_open_record state;
-			struct timeval timeout;
-
-			if (delayed_for_oplocks) {
-				DEBUG(0, ("Trying to delay for oplocks "
-					  "twice\n"));
-				exit_server("exiting");
-			}
-
-			timeout = timeval_set(OPLOCK_BREAK_TIMEOUT*2, 0);
-
-			/* Normally the smbd we asked should respond within
-			 * OPLOCK_BREAK_TIMEOUT seconds regardless of whether
-			 * the client did, give twice the timeout as a safety
-			 * measure here in case the other smbd is stuck
-			 * somewhere else. */
-
-			state.delayed_for_oplocks = True;
-			state.dev = dev;
-			state.inode = inode;
-
-			if (!request_timed_out(request_time, timeout)) {
-				defer_open(lck, request_time, timeout,
-					   &state);
-			}
-
-			talloc_free(lck);
+		/* First pass - send break only on batch oplocks. */
+		if (delay_for_oplocks(lck, fsp, 1, oplock_request)) {
+			schedule_defer_open(lck, request_time);
+			TALLOC_FREE(lck);
+			file_free(fsp);
 			return NULL;
 		}
 
@@ -1381,10 +1383,21 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 					 access_mask, share_access,
 					 create_options, &file_existed);
 
+		if (NT_STATUS_IS_OK(status)) {
+			/* We might be going to allow this open. Check oplock status again. */
+			/* Second pass - send break for both batch or exclusive oplocks. */
+			if (delay_for_oplocks(lck, fsp, 2, oplock_request)) {
+				schedule_defer_open(lck, request_time);
+				TALLOC_FREE(lck);
+				file_free(fsp);
+				return NULL;
+			}
+		}
+
 		if (NT_STATUS_EQUAL(status, NT_STATUS_DELETE_PENDING)) {
 			/* DELETE_PENDING is not deferred for a second */
 			set_saved_ntstatus(status);
-			talloc_free(lck);
+			TALLOC_FREE(lck);
 			file_free(fsp);
 			return NULL;
 		}
@@ -1405,7 +1418,7 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 							  create_options);
 
 				if (fsp_dup) {
-					talloc_free(lck);
+					TALLOC_FREE(lck);
 					file_free(fsp);
 					if (pinfo) {
 						*pinfo = FILE_WAS_OPENED;
@@ -1450,12 +1463,32 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 			 * cope with the braindead 1 second delay.
 			 */
 
-			if (!internal_only_open &&
+			if (!(oplock_request & INTERNAL_OPEN_ONLY) &&
 			    lp_defer_sharing_violations()) {
 				struct timeval timeout;
 				struct deferred_open_record state;
+				int timeout_usecs;
 
-				timeout = timeval_set(0, SHARING_VIOLATION_USEC_WAIT);
+				/* this is a hack to speed up torture tests
+				   in 'make test' */
+				timeout_usecs = lp_parm_int(conn->service,
+							    "smbd","sharedelay",
+							    SHARING_VIOLATION_USEC_WAIT);
+
+				/* This is a relative time, added to the absolute
+				   request_time value to get the absolute timeout time.
+				   Note that if this is the second or greater time we enter
+				   this codepath for this particular request mid then
+				   request_time is left as the absolute time of the *first*
+				   time this request mid was processed. This is what allows
+				   the request to eventually time out. */
+
+				timeout = timeval_set(0, timeout_usecs);
+
+				/* Nothing actually uses state.delayed_for_oplocks
+				   but it's handy to differentiate in debug messages
+				   between a 30 second delay due to oplock break, and
+				   a 1 second delay for share mode conflicts. */
 
 				state.delayed_for_oplocks = False;
 				state.dev = dev;
@@ -1468,7 +1501,7 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 				}
 			}
 
-			talloc_free(lck);
+			TALLOC_FREE(lck);
 			if (fsp_open) {
 				fd_close(conn, fsp);
 				/*
@@ -1511,7 +1544,7 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 
 	if (!fsp_open) {
 		if (lck != NULL) {
-			talloc_free(lck);
+			TALLOC_FREE(lck);
 		}
 		file_free(fsp);
 		return NULL;
@@ -1537,7 +1570,9 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 		dev = fsp->dev;
 		inode = fsp->inode;
 
-		lck = get_share_mode_lock(NULL, dev, inode, fname);
+		lck = get_share_mode_lock(NULL, dev, inode,
+					conn->connectpath,
+					fname);
 
 		if (lck == NULL) {
 			DEBUG(0, ("open_file_ntcreate: Could not get share mode lock for %s\n", fname));
@@ -1570,7 +1605,7 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 
 			defer_open(lck, request_time, timeval_zero(),
 				   &state);
-			talloc_free(lck);
+			TALLOC_FREE(lck);
 			return NULL;
 		}
 
@@ -1607,7 +1642,7 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 		 */
 		if ((SMB_VFS_FTRUNCATE(fsp,fsp->fh->fd,0) == -1) ||
 		    (SMB_VFS_FSTAT(fsp,fsp->fh->fd,psbuf)==-1)) {
-			talloc_free(lck);
+			TALLOC_FREE(lck);
 			fd_close(conn,fsp);
 			file_free(fsp);
 			return NULL;
@@ -1650,34 +1685,31 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 			fsp->oplock_type = NO_OPLOCK;
 		}
 	}
-	set_share_mode(lck, fsp, 0, fsp->oplock_type);
+	set_share_mode(lck, fsp, current_user.ut.uid, 0, fsp->oplock_type);
 
-	if (create_options & FILE_DELETE_ON_CLOSE) {
-		uint32 dosattr= existing_dos_attributes;
-		NTSTATUS result;
-
-		if (info == FILE_WAS_OVERWRITTEN || info == FILE_WAS_CREATED ||
-				info == FILE_WAS_SUPERSEDED) {
-			dosattr = new_dos_attributes;
-		}
-
-		result = can_set_delete_on_close(fsp, True, dosattr);
-
-		if (!NT_STATUS_IS_OK(result)) {
-			/* Remember to delete the mode we just added. */
-			del_share_mode(lck, fsp);
-			talloc_free(lck);
-			fd_close(conn,fsp);
-			file_free(fsp);
-			set_saved_ntstatus(result);
-			return NULL;
-		}
-		lck->delete_on_close = True;
-		lck->modified = True;
-	}
-	
 	if (info == FILE_WAS_OVERWRITTEN || info == FILE_WAS_CREATED ||
 				info == FILE_WAS_SUPERSEDED) {
+
+		/* Handle strange delete on close create semantics. */
+		if (create_options & FILE_DELETE_ON_CLOSE) {
+			NTSTATUS result = can_set_delete_on_close(fsp, True, new_dos_attributes);
+
+			if (!NT_STATUS_IS_OK(result)) {
+				/* Remember to delete the mode we just added. */
+				del_share_mode(lck, fsp);
+				TALLOC_FREE(lck);
+				fd_close(conn,fsp);
+				file_free(fsp);
+				set_saved_ntstatus(result);
+				return NULL;
+			}
+			/* Note that here we set the *inital* delete on close flag,
+			   not the regular one. */
+			set_delete_on_close_token(lck, &current_user.ut);
+			lck->initial_delete_on_close = True;
+			lck->modified = True;
+		}
+	
 		/* Files should be initially set as archive */
 		if (lp_map_archive(SNUM(conn)) ||
 		    lp_store_dos_attributes(SNUM(conn))) {
@@ -1717,7 +1749,7 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 			if (ret == -1 && errno == ENOSYS) {
 				errno = saved_errno; /* Ignore ENOSYS */
 			} else {
-				DEBUG(5, ("open_file_shared: reset "
+				DEBUG(5, ("open_file_ntcreate: reset "
 					  "attributes of file %s to 0%o\n",
 					fname, (unsigned int)new_unx_mode));
 				ret = 0; /* Don't do the fchmod below. */
@@ -1726,7 +1758,7 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 
 		if ((ret == -1) &&
 		    (SMB_VFS_FCHMOD(fsp, fsp->fh->fd, new_unx_mode) == -1))
-			DEBUG(5, ("open_file_shared: failed to reset "
+			DEBUG(5, ("open_file_ntcreate: failed to reset "
 				  "attributes of file %s to 0%o\n",
 				fname, (unsigned int)new_unx_mode));
 	}
@@ -1734,7 +1766,7 @@ files_struct *open_file_ntcreate(connection_struct *conn,
 	/* If this is a successful open, we must remove any deferred open
 	 * records. */
 	del_deferred_open_entry(lck, mid);
-	talloc_free(lck);
+	TALLOC_FREE(lck);
 
 	conn->num_files_open++;
 
@@ -1824,12 +1856,6 @@ files_struct *open_directory(connection_struct *conn,
 		return NULL;
 	}
 
-	if (dir_existed && !S_ISDIR(psbuf->st_mode)) {
-		DEBUG(0,("open_directory: %s is not a directory !\n", fname ));
-		set_saved_ntstatus(NT_STATUS_NOT_A_DIRECTORY);
-		return NULL;
-	}
-
 	switch( create_disposition ) {
 		case FILE_OPEN:
 			/* If directory exists open. If directory doesn't
@@ -1877,7 +1903,6 @@ files_struct *open_directory(connection_struct *conn,
 			DEBUG(5,("open_directory: invalid create_disposition "
 				 "0x%x for directory %s\n",
 				 (unsigned int)create_disposition, fname));
-			file_free(fsp);
 			set_saved_ntstatus(NT_STATUS_INVALID_PARAMETER);
 			return NULL;
 	}
@@ -1944,7 +1969,9 @@ files_struct *open_directory(connection_struct *conn,
 	fsp->is_stat = False;
 	string_set(&fsp->fsp_name,fname);
 
-	lck = get_share_mode_lock(NULL, fsp->dev, fsp->inode, fname);
+	lck = get_share_mode_lock(NULL, fsp->dev, fsp->inode,
+				conn->connectpath,
+				fname);
 
 	if (lck == NULL) {
 		DEBUG(0, ("open_directory: Could not get share mode lock for %s\n", fname));
@@ -1959,27 +1986,30 @@ files_struct *open_directory(connection_struct *conn,
 
 	if (!NT_STATUS_IS_OK(status)) {
 		set_saved_ntstatus(status);
-		talloc_free(lck);
+		TALLOC_FREE(lck);
 		file_free(fsp);
 		return NULL;
 	}
 
-	set_share_mode(lck, fsp, 0, NO_OPLOCK);
+	set_share_mode(lck, fsp, current_user.ut.uid, 0, NO_OPLOCK);
 
+	/* For directories the delete on close bit at open time seems
+	   always to be honored on close... See test 19 in Samba4 BASE-DELETE. */
 	if (create_options & FILE_DELETE_ON_CLOSE) {
 		status = can_set_delete_on_close(fsp, True, 0);
 		if (!NT_STATUS_IS_OK(status)) {
 			set_saved_ntstatus(status);
-			talloc_free(lck);
+			TALLOC_FREE(lck);
 			file_free(fsp);
 			return NULL;
 		}
 
-		lck->delete_on_close = True;
+		set_delete_on_close_token(lck, &current_user.ut);
+		lck->initial_delete_on_close = True;
 		lck->modified = True;
 	}
 
-	talloc_free(lck);
+	TALLOC_FREE(lck);
 
 	/* Change the owner if required. */
 	if ((info == FILE_WAS_CREATED) && lp_inherit_owner(SNUM(conn))) {
@@ -2041,3 +2071,55 @@ files_struct *open_file_stat(connection_struct *conn, char *fname,
 
 	return fsp;
 }
+
+/****************************************************************************
+ Receive notification that one of our open files has been renamed by another
+ smbd process.
+****************************************************************************/
+
+void msg_file_was_renamed(int msg_type, struct process_id src, void *buf, size_t len)
+{
+	files_struct *fsp;
+	char *frm = (char *)buf;
+	SMB_DEV_T dev;
+	SMB_INO_T inode;
+	const char *sharepath;
+	const char *newname;
+	size_t sp_len;
+
+	if (buf == NULL || len < MSG_FILE_RENAMED_MIN_SIZE + 2) {
+                DEBUG(0, ("msg_file_was_renamed: Got invalid msg len %d\n", (int)len));
+                return;
+        }
+
+	/* Unpack the message. */
+	dev = DEV_T_VAL(frm,0);
+	inode = INO_T_VAL(frm,8);
+	sharepath = &frm[16];
+	newname = sharepath + strlen(sharepath) + 1;
+	sp_len = strlen(sharepath);
+
+	DEBUG(10,("msg_file_was_renamed: Got rename message for sharepath %s, new name %s, "
+		"dev %x, inode  %.0f\n",
+		sharepath, newname, (unsigned int)dev, (double)inode ));
+
+	for(fsp = file_find_di_first(dev, inode); fsp; fsp = file_find_di_next(fsp)) {
+		if (memcmp(fsp->conn->connectpath, sharepath, sp_len) == 0) {
+	                DEBUG(10,("msg_file_was_renamed: renaming file fnum %d from %s -> %s\n",
+				fsp->fnum, fsp->fsp_name, newname ));
+			string_set(&fsp->fsp_name, newname);
+		} else {
+			/* TODO. JRA. */
+			/* Now we have the complete path we can work out if this is
+			   actually within this share and adjust newname accordingly. */
+	                DEBUG(10,("msg_file_was_renamed: share mismatch (sharepath %s "
+				"not sharepath %s) "
+				"fnum %d from %s -> %s\n",
+				fsp->conn->connectpath,
+				sharepath,
+				fsp->fnum,
+				fsp->fsp_name,
+				newname ));
+		}
+        }
+}
