@@ -54,7 +54,7 @@ static NTSTATUS cmd_populate(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int arg
 	}
 	c = argv[1][0];
 	size = atoi(argv[2]);
-	vfs->data = (char *)talloc(mem_ctx, size);
+	vfs->data = TALLOC_ARRAY(mem_ctx, char, size);
 	if (vfs->data == NULL) {
 		printf("populate: error=-1 (not enough memory)");
 		return NT_STATUS_UNSUCCESSFUL;
@@ -129,7 +129,7 @@ static NTSTATUS cmd_opendir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc
 		return NT_STATUS_OK;
 	}
 
-	vfs->currentdir = SMB_VFS_OPENDIR(vfs->conn, argv[1]);
+	vfs->currentdir = SMB_VFS_OPENDIR(vfs->conn, argv[1], NULL, 0);
 	if (vfs->currentdir == NULL) {
 		printf("opendir error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
@@ -142,7 +142,7 @@ static NTSTATUS cmd_opendir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc
 
 static NTSTATUS cmd_readdir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
-	struct dirent *dent;
+	SMB_STRUCT_DIRENT *dent;
 
 	if (vfs->currentdir == NULL) {
 		printf("readdir: error=-1 (no open directory)\n");
@@ -284,9 +284,10 @@ static NTSTATUS cmd_open(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, c
 		return NT_STATUS_UNSUCCESSFUL;
 	}
 
-	vfs->files[fd] = (struct files_struct *)malloc(sizeof(struct files_struct));
-	vfs->files[fd]->fsp_name = strdup(argv[1]);
-	vfs->files[fd]->fd = fd;
+	vfs->files[fd] = SMB_MALLOC_P(struct files_struct);
+	vfs->files[fd]->fsp_name = SMB_STRDUP(argv[1]);
+	vfs->files[fd]->fh = SMB_MALLOC_P(struct fd_handle);
+	vfs->files[fd]->fh->fd = fd;
 	vfs->files[fd]->conn = vfs->conn;
 	printf("open: fd=%d\n", fd);
 	return NT_STATUS_OK;
@@ -345,6 +346,7 @@ static NTSTATUS cmd_close(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc,
 		printf("close: ok\n");
 
 	SAFE_FREE(vfs->files[fd]->fsp_name);
+	SAFE_FREE(vfs->files[fd]->fh);
 	SAFE_FREE(vfs->files[fd]);
 	vfs->files[fd] = NULL;
 	return NT_STATUS_OK;
@@ -364,7 +366,7 @@ static NTSTATUS cmd_read(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, c
 	/* do some error checking on these */
 	fd = atoi(argv[1]);
 	size = atoi(argv[2]);
-	vfs->data = (char *)talloc(mem_ctx, size);
+	vfs->data = TALLOC_ARRAY(mem_ctx, char, size);
 	if (vfs->data == NULL) {
 		printf("read: error=-1 (not enough memory)");
 		return NT_STATUS_UNSUCCESSFUL;
@@ -522,8 +524,12 @@ static NTSTATUS cmd_stat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, c
 	else if (S_ISLNK(st.st_mode)) printf("  Symbolic Link\n");
 	else if (S_ISSOCK(st.st_mode)) printf("  Socket\n");
 	printf("  Size: %10u", (unsigned int)st.st_size);
+#ifdef HAVE_STAT_ST_BLOCKS
 	printf(" Blocks: %9u", (unsigned int)st.st_blocks);
+#endif
+#ifdef HAVE_STAT_ST_BLKSIZE
 	printf(" IO Block: %u\n", (unsigned int)st.st_blksize);
+#endif
 	printf("  Device: 0x%10x", (unsigned int)st.st_dev);
 	printf(" Inode: %10u", (unsigned int)st.st_ino);
 	printf(" Links: %10u\n", (unsigned int)st.st_nlink);
@@ -534,8 +540,6 @@ static NTSTATUS cmd_stat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, c
 	printf("  Modify: %s", ctime(&(st.st_mtime)));
 	printf("  Change: %s", ctime(&(st.st_ctime)));
 
-	SAFE_FREE(pwd);
-	SAFE_FREE(grp);
 	return NT_STATUS_OK;
 }
 
@@ -586,8 +590,12 @@ static NTSTATUS cmd_fstat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc,
 	else if (S_ISLNK(st.st_mode)) printf("  Symbolic Link\n");
 	else if (S_ISSOCK(st.st_mode)) printf("  Socket\n");
 	printf("  Size: %10u", (unsigned int)st.st_size);
+#ifdef HAVE_STAT_ST_BLOCKS
 	printf(" Blocks: %9u", (unsigned int)st.st_blocks);
+#endif
+#ifdef HAVE_STAT_ST_BLKSIZE
 	printf(" IO Block: %u\n", (unsigned int)st.st_blksize);
+#endif
 	printf("  Device: 0x%10x", (unsigned int)st.st_dev);
 	printf(" Inode: %10u", (unsigned int)st.st_ino);
 	printf(" Links: %10u\n", (unsigned int)st.st_nlink);
@@ -598,8 +606,6 @@ static NTSTATUS cmd_fstat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc,
 	printf("  Modify: %s", ctime(&(st.st_mtime)));
 	printf("  Change: %s", ctime(&(st.st_ctime)));
 
-	SAFE_FREE(pwd);
-	SAFE_FREE(grp);
 	return NT_STATUS_OK;
 }
 
@@ -638,8 +644,12 @@ static NTSTATUS cmd_lstat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc,
 	else if (S_ISLNK(st.st_mode)) printf("  Symbolic Link\n");
 	else if (S_ISSOCK(st.st_mode)) printf("  Socket\n");
 	printf("  Size: %10u", (unsigned int)st.st_size);
+#ifdef HAVE_STAT_ST_BLOCKS
 	printf(" Blocks: %9u", (unsigned int)st.st_blocks);
+#endif
+#ifdef HAVE_STAT_ST_BLKSIZE
 	printf(" IO Block: %u\n", (unsigned int)st.st_blksize);
+#endif
 	printf("  Device: 0x%10x", (unsigned int)st.st_dev);
 	printf(" Inode: %10u", (unsigned int)st.st_ino);
 	printf(" Links: %10u\n", (unsigned int)st.st_nlink);
@@ -650,8 +660,6 @@ static NTSTATUS cmd_lstat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc,
 	printf("  Modify: %s", ctime(&(st.st_mtime)));
 	printf("  Change: %s", ctime(&(st.st_ctime)));
 	
-	SAFE_FREE(pwd);
-	SAFE_FREE(grp);
 	return NT_STATUS_OK;
 }
 
