@@ -74,7 +74,7 @@ static int net_status_sessions(int argc, const char **argv)
 			   TDB_DEFAULT, O_RDONLY, 0);
 
 	if (tdb == NULL) {
-		d_printf("%s not initialised\n", lock_path("sessionid.tdb"));
+		d_fprintf(stderr, "%s not initialised\n", lock_path("sessionid.tdb"));
 		return -1;
 	}
 
@@ -104,7 +104,7 @@ static int show_share(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf,
 	d_printf("%-10.10s   %s   %-12s  %s",
 	       crec.name,procid_str_static(&crec.pid),
 	       crec.machine,
-	       asctime(localtime(&crec.start)));
+	       time_to_asc(&crec.start));
 
 	return 0;
 }
@@ -130,6 +130,10 @@ static int collect_pid(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf,
 
 	ids->num_entries += 1;
 	ids->entries = SMB_REALLOC_ARRAY(ids->entries, struct sessionid, ids->num_entries);
+	if (!ids->entries) {
+		ids->num_entries = 0;
+		return 0;
+	}
 	ids->entries[ids->num_entries-1] = sessionid;
 
 	return 0;
@@ -169,7 +173,7 @@ static int show_share_parseable(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf,
 		 guest ? "" : gidtoname(ids->entries[i].gid),
 		 crec.machine, 
 		 guest ? "" : ids->entries[i].hostname,
-		 asctime(localtime(&crec.start)));
+		 time_to_asc(&crec.start));
 
 	return 0;
 }
@@ -186,7 +190,7 @@ static int net_status_shares_parseable(int argc, const char **argv)
 			   TDB_DEFAULT, O_RDONLY, 0);
 
 	if (tdb == NULL) {
-		d_printf("%s not initialised\n", lock_path("sessionid.tdb"));
+		d_fprintf(stderr, "%s not initialised\n", lock_path("sessionid.tdb"));
 		return -1;
 	}
 
@@ -197,9 +201,9 @@ static int net_status_shares_parseable(int argc, const char **argv)
 			   TDB_DEFAULT, O_RDONLY, 0);
 
 	if (tdb == NULL) {
-		d_printf("%s not initialised\n", lock_path("connections.tdb"));
-		d_printf("This is normal if no SMB client has ever connected "
-			 "to your server.\n");
+		d_fprintf(stderr, "%s not initialised\n", lock_path("connections.tdb"));
+		d_fprintf(stderr, "This is normal if no SMB client has ever "
+			 "connected to your server.\n");
 		return -1;
 	}
 
@@ -226,10 +230,10 @@ static int net_status_shares(int argc, const char **argv)
 				   TDB_DEFAULT, O_RDONLY, 0);
 
 		if (tdb == NULL) {
-			d_printf("%s not initialised\n",
+			d_fprintf(stderr, "%s not initialised\n",
 				 lock_path("connections.tdb"));
-			d_printf("This is normal if no SMB client has ever "
-				 "connected to your server.\n");
+			d_fprintf(stderr, "This is normal if no SMB client has "
+				 "ever connected to your server.\n");
 			return -1;
 		}
 
